import os

from cpl.application import ApplicationABC
from cpl.configuration import ConfigurationABC
from cpl.console import Console
from cpl.dependency_injection import ServiceProviderABC

from lexer.abc.lexer_abc import LexerABC
from runtime.abc.runtime_service_abc import RuntimeServiceABC


class Application(ApplicationABC):

    def __init__(self, config: ConfigurationABC, services: ServiceProviderABC):
        ApplicationABC.__init__(self, config, services)

        self._lexer: LexerABC = services.get_service(LexerABC)
        self._runtime: RuntimeServiceABC = services.get_service(RuntimeServiceABC)

        self._path = config.get_configuration('p')

    def _interpret(self, line: str):
        tokens = self._lexer.tokenize(line)

        line.replace("\n", "").replace("\t", "")
        Console.write_line(f'\nLINE: {line}')
        tokens.for_each(lambda t: Console.write_line(t.type, t.value))

    def _console(self):
        i = 0
        while True:
            self._runtime.line_count = i + 1
            self._interpret(input('> '))
            i += 1

    def _files(self):
        if not os.path.isdir(self._path):
            raise FileNotFoundError(self._path)

        # r=root, d=directories, f=files
        for r, d, f in os.walk(self._path):
            for file in f:
                if file.endswith('.cc'):
                    self._read_file(os.path.join(r, file))

    def _read_file(self, file: str):
        if not os.path.isfile(file):
            raise FileNotFoundError
            # self.__utils.runtime_error(Error(ErrorCodes.FileNotFound))

        if not file.endswith('.cc'):
            raise Exception('Wrong file type')
            # self.__utils.runtime_error(Error(ErrorCodes.WrongFileType))

        f = open(file, 'r', encoding='utf-8').readlines()
        for i in range(0, len(f)):
            self._runtime.line_count = i + 1
            self._interpret(f[i])

    def configure(self):
        pass

    def main(self):
        if self._path is None:
            self._console()
            return

        if os.path.isfile(self._path):
            self._read_file(self._path)
        else:
            self._files()
